diff --git a/common/frame.c b/common/frame.c
index a51dc29..de305d1 100644
--- a/common/frame.c
+++ b/common/frame.c
@@ -35,6 +35,7 @@ x264_frame_t *x264_frame_new( x264_t *h )
     int i_stride, i_width, i_lines;
     int i_padv = PADV << h->param.b_interlaced;
     int luma_plane_size;
+    int chroma_plane_size;
     int align = h->param.cpu&X264_CPU_CACHELINE_64 ? 64 : h->param.cpu&X264_CPU_CACHELINE_32 ? 32 : 16;
 
     if( !frame ) return NULL;
@@ -49,15 +50,16 @@ x264_frame_t *x264_frame_new( x264_t *h )
     frame->i_plane = 3;
     for( i = 0; i < 3; i++ )
     {
-        frame->i_stride[i] = i_stride >> !!i;
+        frame->i_stride[i] = ALIGN( i_stride >> !!i, 16 );
         frame->i_width[i] = i_width >> !!i;
         frame->i_lines[i] = i_lines >> !!i;
     }
 
     luma_plane_size = (frame->i_stride[0] * ( frame->i_lines[0] + 2*i_padv ));
+    chroma_plane_size = (frame->i_stride[1] * ( frame->i_lines[1] + 2*i_padv ));
     for( i = 1; i < 3; i++ )
     {
-        CHECKED_MALLOC( frame->buffer[i], luma_plane_size/4 );
+        CHECKED_MALLOC( frame->buffer[i], chroma_plane_size );
         frame->plane[i] = frame->buffer[i] + (frame->i_stride[i] * i_padv + PADH)/2;
     }
     /* all 4 luma planes allocated together, since the cacheline split code
diff --git a/common/ppc/pixel.c b/common/ppc/pixel.c
index 0476814..28c1ce2 100644
--- a/common/ppc/pixel.c
+++ b/common/ppc/pixel.c
@@ -1672,19 +1672,20 @@ static int x264_pixel_var_8x8_altivec( uint8_t *pix, int i_stride )
     vec_u32_t sqr_v = zero_u32v;
     vec_u32_t sum_v = zero_u32v;
 
-    vec_u8_t perm0 = vec_lvsl( 0, pix );
-    vec_u8_t perm1 = vec_lvsl( 0, pix+i_stride );
+    static const vec_u8_t perm_tab[] = {
+        CV(0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,  /* pix=mod16, i_stride=mod16 */
+           0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17),
+        CV(0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,  /* pix=mod16, i_stride=mod8  */
+           0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F),
+    };
+    vec_u8_t perm = perm_tab[ ((i_stride & 8) >> 3) ];
 
     int y;
     for( y = 0; y < 8; y+=2 )
     {
         vec_u8_t pix0_v = vec_ld(0, pix);
         vec_u8_t pix1_v = vec_ld(i_stride, pix);
-        pix0_v = vec_perm(pix0_v, pix0_v, perm0);
-        pix1_v = vec_perm(pix1_v, pix1_v, perm1);
-
-        vec_u8_t pix_v = vec_mergeh(pix0_v, pix1_v);
-
+        vec_u8_t pix_v = vec_perm(pix0_v, pix1_v, perm);
         sum_v = vec_sum4s(pix_v, sum_v);
         sqr_v = vec_msum(pix_v, pix_v, sqr_v);
 